@@ -12,32 +12,35 @@ import select
 import threading
 from . import adns
 from ..protocols.http2 import H2_PROTOCOLS
-from ..lib.athreads.fifo import await_fifo
-from ..lib.ssl_ import resolve_cert_reqs, resolve_ssl_version, create_urllib3_context
+from ..athreads.fifo import await_fifo
+from rs4.ssl_ import resolve_cert_reqs, resolve_ssl_version, create_urllib3_context
 from collections import deque
 from ..protocols.http import respcodes
 import random
 
 DEBUG = 0
-	
+DEFAULT_ZOMBIE_TIMEOUT = 60
+DEFAULT_KEEP_ALIVE = 2
+
 class SocketPanic (Exception): pass
 class TimeOut (Exception): pass
 
 class AsynConnect (asynchat.async_chat):
 	ac_in_buffer_size = 65535
 	ac_out_buffer_size = 65535
-	zombie_timeout = 10
-	keep_alive = 10	
+	zombie_timeout = DEFAULT_ZOMBIE_TIMEOUT
+	keep_alive = DEFAULT_KEEP_ALIVE	
 	request_count = 0
 	active = 0
 	fifo_class = deque
 	keep_connect = True
+	is_ssl = False
 	
 	def __init__ (self, address, lock = None, logger = None):
 		self.address = address
 		self.lock = lock
 		self.logger = logger
-		self._cv = threading.Condition ()		
+		self._cv = threading.Condition ()
 		self.__sendlock = None
 		self.__no_more_request = False
 		self.set_event_time ()
@@ -69,7 +72,7 @@ class AsynConnect (asynchat.async_chat):
 		return new_asyncon
 	
 	def set_backend (self, backend_keep_alive = 10):
-		#self.backend = True
+		self.backend = True
 		self.keep_alive = backend_keep_alive
 		
 	def set_auth (self, auth):
@@ -106,7 +109,7 @@ class AsynConnect (asynchat.async_chat):
 		handler, self.handller = self.handler, None
 		keep_active = False
 		try:
-			keep_active = handler.connection_closed (self.errcode, self.errmsg)
+			keep_active = handler.connection_closed (self.errcode, self.errmsg)			
 		except: 
 			self.trace ()
 		
@@ -184,7 +187,7 @@ class AsynConnect (asynchat.async_chat):
 		
 	def clean_shutdown_control (self, phase, time_in_this_phase):	
 		self.__no_more_request = True
-		if self.isactive () or (self.handler and self.handler.working ()):
+		if self.isactive () or (self.handler and self.handler.working ()):		
 			return 1
 		else:				
 			self.handle_close (712, "Controlled Shutdown")
@@ -259,7 +262,7 @@ class AsynConnect (asynchat.async_chat):
 			except:	self.handle_error (714)
 			return
 		
-		ipaddr = answer and answer [-1]["data"] or None
+		ipaddr = answer [-1]["data"]
 		#print (self.handler.request.meta ['sid'], ipaddr, 'continue_connect...')
 		if not ipaddr:			
 			return self.handle_close (704)			
@@ -290,8 +293,7 @@ class AsynConnect (asynchat.async_chat):
 	
 	def send (self, data):
 		try:
-			numsent = self.socket.send (data)			
-			#print ('-------', numsent)
+			numsent = self.socket.send (data)
 			if numsent:
 				self.set_event_time ()
 			return numsent	
@@ -315,7 +317,7 @@ class AsynConnect (asynchat.async_chat):
 				raise
 	
 	def close_if_over_keep_live (self):		
-		if time.time () - self.event_time > max (self.keep_alive - 10, 5):
+		if time.time () - self.event_time > self.keep_alive:
 			self.disconnect ()
 		
 	def set_timeout (self, timeout = 10):
@@ -334,7 +336,7 @@ class AsynConnect (asynchat.async_chat):
 	
 	def handle_error (self, code = 701):
 		self.trace ()
-		self.handle_close(code)
+		self.handle_close (code)
 	
 	def handle_timeout (self):		
 		self.handle_close (702)
@@ -444,19 +446,23 @@ class AsynConnect (asynchat.async_chat):
 
 
 class AsynSSLConnect (AsynConnect):	
-	ac_negotiate_http2 = True
+	is_ssl = True
+
+	def __init__ (self, address, lock = None, logger = None):
+		AsynConnect.__init__ (self, address, lock, logger)
+		self.ac_negotiate_http2 = True
 	
 	def negotiate_http2 (self, flag):
 		self.ac_negotiate_http2 = flag
 		
-	def handshake (self):
+	def handshake (self):		
 		if not self._handshaking:
 			err = self.socket.getsockopt(socket.SOL_SOCKET, socket.SO_ERROR)
 			if err != 0:
 				raise OSError(err, asyncore._strerror(err))
 				
 			ssl_context = create_urllib3_context(ssl_version=resolve_ssl_version(None), cert_reqs=resolve_cert_reqs(None))
-			if self.ac_negotiate_http2:
+			if self.ac_negotiate_http2:								
 				try: ssl_context.set_alpn_protocols (H2_PROTOCOLS)
 				except AttributeError: ssl_context.set_npn_protocols (H2_PROTOCOLS)								
 			self.socket = ssl_context.wrap_socket (self.socket, do_handshake_on_connect = False, server_hostname = self.address [0])			
@@ -482,7 +488,7 @@ class AsynSSLConnect (AsynConnect):
 			if not self._handshaked and not self.handshake ():
 				return
 		except:
-			return self.handle_error (713)
+			return self.handle_error (713)		
 		AsynConnect.handle_connect_event (self)		
 		
 	def recv (self, buffer_size):
@@ -491,7 +497,7 @@ class AsynSSLConnect (AsynConnect):
 			return b''
 			
 		try:
-			data = self.socket.recv (buffer_size)						
+			data = self.socket.recv (buffer_size)			
 			if not data:				
 				self.handle_close (700, "Connection closed unexpectedly")
 				return b''
@@ -514,13 +520,13 @@ class AsynSSLConnect (AsynConnect):
 			else:
 				raise
 
-	def send (self, data):
+	def send (self, data):		
 		if self._closed:
 			# usually handshaking failure, already handled exception
 			return
 			
 		try:
-			numsent = self.socket.send (data)			
+			numsent = self.socket.send (data)	
 			if numsent:
 				self.set_event_time ()
 			return numsent	
@@ -536,13 +542,18 @@ class AsynSSLConnect (AsynConnect):
 
 
 class AsynSSLProxyConnect (AsynSSLConnect, AsynConnect):
+	is_ssl = True
+
+	def __init__ (self, address, lock = None, logger = None):
+		AsynConnect.__init__ (self, address, lock, logger)
+
 	def handle_connect_event (self):
 		if self.established:
 			AsynSSLConnect.handle_connect_event (self)
 		else:	
 			AsynConnect.handle_connect_event (self)
 	
-	def recv (self, buffer_size):	
+	def recv (self, buffer_size):		
 		if self._handshaked or self._handshaking:
 			return AsynSSLConnect.recv (self, buffer_size)				
 		else:
