@@ -19,35 +19,55 @@ else:
 	import asyncore
 	from . import dbconnect
 
-	from psycopg2.extensions import POLL_OK, POLL_WRITE, POLL_READ
+	from psycopg2.extensions import POLL_OK, POLL_READ, POLL_WRITE
 	_STATE_OK = (POLL_OK, POLL_WRITE, POLL_READ)
-		
+	_STATE_RETRY = -1
+	_STATE_IGNORE = -2
+	REREY_TEST = False	
+
 	class AsynConnect (dbconnect.AsynDBConnect, asyncore.dispatcher):
 		def __init__ (self, address, params = None, lock = None, logger = None):
 			dbconnect.AsynDBConnect.__init__ (self, address, params, lock, logger)			
-			self.conn = None
 			self.cur = None			
 			asyncore.dispatcher.__init__ (self)
-			
-		def check_state (self, state):
-			if state not in (_STATE_OK):				
+
+		def retry (self):
+			if self.request is None:
+				return
+			self.logger ("[warn] closed psycopg2 connection, retrying...")
+			self.disconnect ()
+			request, self.request = self.request, None
+			self.execute (request)
+			return _STATE_RETRY
+
+		def check_state (self, state):			
+			if state == _STATE_RETRY:				
+				return
+			if state not in (_STATE_OK):
 				self.logger ("[warn] psycopg2.poll() returned %s" % state)
 				self.handle_close ()
-		
+			
 		def poll (self):
 			# 2 cases, if on requesting raise immediatly, else handle silently
 			try:
-				return self.socket.poll ()
-			except psycopg2.OperationalError:				
+				if REREY_TEST and self.writable () and self.request.retry_count == 0:
+					#raise psycopg2.InterfaceError
+					self.disconnect ()
+				return self.socket.poll ()			
+			except (psycopg2.OperationalError, psycopg2.InterfaceError):							
 				if self.request:
-					raise
-				# else usually timeout	
+					if self.request.retry_count == 0:
+						self.request.retry_count += 1
+						return self.retry ()
+					else:
+						raise
+				self.logger.trace ()				
 			except:
+				# else usually timeout	
 				if self.request:
 					raise
-				# logging
 				self.logger.trace ()
-			return -1
+			return _STATE_IGNORE
 			
 		def writable (self):			
 			return self.out_buffer or not self.connected
@@ -84,16 +104,13 @@ else:
 				self.handle_write ()
 		
 		def handle_expt (self):
-			self.handle_close (psycopg2.OperationalError, "socket panic")
+			self.handle_close (psycopg2.OperationalError ("Socket Panic"))
+			
+		def handle_connect (self):			
+			self.create_cursor ()
 			
-		def handle_connect (self):
-			self.del_channel ()
-			self.conn = self.socket
-			self.cur = self.conn.cursor()		
-			self.set_socket (self.cur.connection)
-					
 		def handle_read (self):
-			state = self.poll ()
+			state = self.poll ()			
 			if self.cur and state == POLL_OK:
 				self.set_event_time ()
 				self.has_result = True
@@ -103,8 +120,10 @@ else:
 				
 		def handle_write (self):
 			state = self.poll ()
-			if self.cur and state == POLL_OK:
-				self.set_event_time ()				
+			if state == POLL_OK:
+				if not self.cur:
+					self.create_cursor ()				
+				self.set_event_time ()
 				self.cur.execute (self.out_buffer)
 				self.out_buffer = ""
 			else:				
@@ -113,41 +132,45 @@ else:
 		#-----------------------------------
 		# Overriden
 		#-----------------------------------
+		def create_cursor (self):
+			if self.cur is None:				
+				self.cur = self.socket.cursor ()						
+
+		def close_cursor (self):
+			if self.cur:
+				self.cur.close ()
+				self.cur = None				
+
 		def close_case (self):			
 			if self.request:
-				if self.has_result and self.cur.description:					
-					self.request.handle_result (self.cur.description, self.exception_class, self.exception_str, self.fetchall ())					
-				else:
-					self.request.handle_result (None, self.exception_class, self.exception_str, None)
-					self.has_result = False
-				self.request = None
+				description, data = self.cur and self.cur.description or None, None
+				if description:
+					try:
+						data = self.fetchall ()
+					except:
+						self.logger.trace ()
+						self.expt = asyncore.compact_traceback () [2]						
+						data = None
+				self.request.handle_result (description or None, self.expt, data)					
+				self.request = None				
+			# self.disconnect ()
+			self.close_cursor ()
 			self.set_active (False)
 			
-		def empty_cursor (self):
-			if self.has_result:
+		def empty_cursor (self):		
+			if self.cur and self.cur.description:
 				try:
 					self.fetchall ()
 				except psycopg2.ProgrammingError:
 					pass				
-		
+	
 		def fetchall (self):
-			result = self.cur.fetchall ()
-			self.has_result = False
-			return result
+			return self.cur.fetchall ()				
 							
-		def close (self, deactive = 1):
-			if self.cur:
-				try:
-					self.cur.close ()
-				except psycopg2.ProgrammingError:
-					pass						
-				self.cur = None
-			if self.conn:	
-				self.conn.close ()
-				self.conn = None	
-				
-			dbconnect.AsynDBConnect.close (self, deactive)
+		def close (self, deactive = 1):					
 			asyncore.dispatcher.close (self)
+			self.close_cursor ()
+			dbconnect.AsynDBConnect.close (self, deactive)			
 			
 		def connect (self, force = 0):
 			host, port = self.address		
@@ -157,25 +180,35 @@ else:
 				password = self.password,
 				host = host,
 				port = port,
-				async = 1
-			)
-			self.set_socket (sock)
+				async_ = 1
+			)			
+			self.set_socket (sock)			
 		
-		def end_tran (self):				
+		def end_tran (self):			
 			if not self.backend:
-				self.del_channel ()
-
-		def begin_tran (self, request):			
-			dbconnect.AsynDBConnect.begin_tran (self, request)
-			self.out_buffer = request.params [0]			
-			
-		def execute (self, request):			
-			self.begin_tran (request)			
-			if not self.connected:
-				self.connect ()				
+				self.del_channel ()				
+		
+		def _compile (self, request):
+			sql = request.params [0]
+			if isinstance (sql, (list, tuple)):
+				sql = ";\n".join (map (str, sql)) + ";"
+			try:
+				sql = sql.strip ()
+			except AttributeError:				
+				raise dbconnect.SQLError ("Invalid SQL")
+			if not sql:
+				raise dbconnect.SQLError ("Empty SQL")
+			return sql
+		
+		def execute (self, request):		
+			dbconnect.DBConnect.begin_tran (self, request)
+			self.out_buffer = self._compile (request)
+			if not self.connected and not self.connecting:				
+				self.connect ()
 			else:
 				state = self.poll ()
 				if state != POLL_OK:
 					self.reconnect ()
-				elif not self.backend:
-					self.add_channel ()
+				else:
+					self.create_cursor ()				
+				
\ No newline at end of file
