@@ -3,15 +3,19 @@ import re
 import time 
 import sys 
 import threading
+from ..client.asynconnect import DEFAULT_ZOMBIE_TIMEOUT, DEFAULT_KEEP_ALIVE
 
 DEBUG = False
 
 class OperationalError (Exception):
 	pass
+
+class SQLError (Exception):
+	pass
 	
 class DBConnect:
-	zombie_timeout = 120
-	keep_alive = 120
+	zombie_timeout = DEFAULT_ZOMBIE_TIMEOUT
+	keep_alive = DEFAULT_KEEP_ALIVE
 	
 	def __init__ (self, address, params = None, lock = None, logger = None):
 		self.address = address
@@ -38,7 +42,6 @@ class DBConnect:
 		
 		# need if there's not any request yet
 		self.active = 0
-		self.retried = 0
 		self.request = None
 		self.has_result = False
 		self.__history = []
@@ -47,12 +50,12 @@ class DBConnect:
 		self.set_event_time ()
 	
 	def close_if_over_keep_live (self):
-		if time.time () - self.event_time > max (self.keep_alive - 10, 5):			
-			self.disconnect ()
+		if time.time () - self.event_time > self.keep_alive:		
+			self.disconnect ()			
 			
 	def set_backend (self, backend_keep_alive = 10):
-		#self.backend = True
-		self.keep_alive = backend_keep_alive
+		self.backend = True
+		self.keep_alive = backend_keep_alive		
 		
 	def duplicate (self):
 		new_asyncon = self.__class__ (self.address, self.params, self.lock, self.logger)
@@ -69,11 +72,13 @@ class DBConnect:
 		self.close ()
 		
 	def close (self, deactive = 1):
-		self.request = None
-		addr = type (self.address) is tuple and ("%s:%d" % self.address) or str (self.address)
-		self.logger ("[info] .....dbo %s has been closed" % addr)	
+		addr = type (self.address) is tuple and ("%s:%d" % self.address) or str (self.address)		
 		if deactive:
+			self.logger ("[info] ..dbo %s has been closed" % addr)			
 			self.set_active (False)
+			self.request = None
+		else:
+			self.logger ("[info] ..dbo %s was disconnected or keep alive timeout" % addr)	
 					
 	def get_history (self):
 		return self.__history
@@ -109,17 +114,14 @@ class DBConnect:
 	
 	def disconnect (self):
 		# keep request, just close temporary
-		request = self.request
-		self.close (deactive = 0)
-		self.request = request
+		self.close (deactive = 0)		
 	
 	def close_case (self):
 		raise NotImplementedError
 			
 	def set_active (self, flag, nolock = False):
 		if not flag: 
-			self.set_timeout (self.keep_alive)
-			self.retried = 0
+			self.set_timeout (self.keep_alive)			
 			
 		if flag:
 			flag = time.time ()
@@ -163,23 +165,17 @@ class DBConnect:
 		self.zombie_timeout = timeout
 		
 	def handle_timeout (self):
-		self.handle_close (OperationalError, "Operation Timeout")
+		self.handle_close (OperationalError ("Operation Timeout"))
 	
 	def handle_error (self):
-		dummy, exception_class, exception_str, tbinfo = asyncore.compact_traceback()
+		dummy, t, v, info = asyncore.compact_traceback ()
 		self.has_result = False		
 		self.logger.trace ()
-		self.handle_close (exception_class, exception_str)
+		self.handle_close (v)
 	
-	def handle_close (self, expt = None, msg = ""):
-		if not expt and not self.retried:			
-			self.disconnect ()
-			self.retried = 1
-			self.execute (self.request)			
-			return
-		
-		if self.exception_class is None:	
-			self.exception_class, self.exception_str = expt, msg		
+	def handle_close (self, expt = None):
+		if self.expt is None:	
+			self.expt = expt
 		self.close_case_with_end_tran ()
 		self.close ()
 	
@@ -204,20 +200,15 @@ class DBConnect:
 						
 	def begin_tran (self, request):
 		if self.__no_more_request:			
-			try: 
-				raise OperationalError ("Entered Shutdown Process")
-			except: 
-				self.handle_error ()
-				return
-		
+			raise OperationalError ("Entered Shutdown Process")			
 		self.request = request	
 		self.__history = []
+		self.out_buffer = ''
 		self.has_result = False
-		self.exception_str = ""
-		self.exception_class = None		
+		self.expt = None		
 		self.execute_count += 1
 		self.close_if_over_keep_live ()
-		self.set_event_time ()
+		self.set_event_time ()		
 		
 	def execute (self, request):		
 		self.begin_tran (request)		
