@@ -34,6 +34,8 @@ class DBPool (socketpool.SocketPool):
 			con_class = asynmongo.AsynConnect
 		elif dbtype == DB_PGSQL:
 			con_class = asynpsycopg2.AsynConnect
+		else:
+			raise ValueError ("Unknown database type: {}".format (dbtype))	
 		asyncon = con_class ((host, port), params, self.lock, self.logger)
 		self.backend and asyncon.set_backend ()
 		return asyncon
@@ -61,7 +63,7 @@ def cleanup ():
 
 if __name__ == "__main__":
 	from skitai import lifetime
-	from aquests.lib import logger
+	from rs4 import logger
 	from aquests.server.threads import trigger
 	
 	trigger.start_trigger ()
